package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.dto.OrderDetailDto;
import com.junmp.jyzb.api.bean.dto.OrderDto;
import com.junmp.jyzb.api.bean.dto.OrderMainDto;
import com.junmp.jyzb.api.bean.query.OrderMainReq;
import com.junmp.jyzb.api.bean.query.PolicemanFingerReq;
import com.junmp.jyzb.api.bean.req.DetailListReq;
import com.junmp.jyzb.api.bean.req.UpdateOrderReq;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.api.exception.enums.OrderExceptionEnum;
import com.junmp.jyzb.entity.*;
import com.junmp.jyzb.mapper.OrderMainMapper;
import com.junmp.jyzb.service.OrderDetailService;
import com.junmp.jyzb.service.OrderNumService;
import com.junmp.jyzb.service.OrderService;
import com.junmp.jyzb.service.PubOrgService;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import com.junmp.v2.db.api.factory.PageFactory;
import com.junmp.v2.db.api.factory.PageResultFactory;
import com.junmp.v2.db.api.page.PageResult;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.core.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;


@Service
public class OrderServiceImpl  extends ServiceImpl<OrderMainMapper, OrderMain> implements OrderService {

    @Resource
    private OrderDetailService orderDetailService;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Resource
    private RabbitAdmin rabbitAdmin;

    @Resource
    private OrderNumService orderNumService;

    @Resource
    private PubOrgService pubOrgService;



    @Transactional(rollbackFor = Exception.class)
    @Override
    public String AddOrder(UpdateOrderReq req) {
        OrderMain order = new OrderMain();
        BeanPlusUtil.copyProperties(req, order);
        //设置单据单号（订单号）
        OrderNum orderNum = setOrderCode(req);
        String codeValue=String.format("%04d",orderNum.getNum());
        order.setOrderCode(orderNum.getBussinessType()+"-"+orderNum.getOutInType()+"-"+orderNum.getYear()+"-"+codeValue);
        //设置总价格和应出入库数量
        //设置id
        String id=UUID.randomUUID().toString();
        order.setId(id);
        Integer sumNum=0;
        BigDecimal priceTotal=new BigDecimal(0);
        List<OrderDetail> detailList = new ArrayList<>();
        //通过遍历批量保存详细信息
        for (DetailListReq listReq:req.getDetailList()) {
            OrderDetail detail=new OrderDetail();
            BeanPlusUtil.copyProperties(listReq, detail);
            detail.setOrderId(id);
            //设置单据类型
            detail.setType(order.getOrderType());
            //将数量和价格进行累加
            sumNum += detail.getPlanNum();
            priceTotal = priceTotal.add(BigDecimal.valueOf(detail.getPlanNum()).multiply(detail.getPrice())) ;
            detailList.add(detail);
        }
        orderDetailService.saveBatch(detailList);
       //设置总价格和总数量
        order.setPrice(priceTotal);
        order.setInventoryQuantity(sumNum);
        //保存
        this.save(order);
        return order.getId();
    }


    //设置单据单号（并且对order_num表进行新增或者更新）
    public OrderNum setOrderCode(UpdateOrderReq req){
        //设置采购单号，需要先判断改组织机构的采购单号是否存在，如果存在则将数量进行增加，如果不存在则新增一条对应的数据
        OrderNum orderNum = new OrderNum();
        //获取年份，获取业务出入状态，获取组织机构id，获取业务类型
        orderNum.setYear(LocalDateTime.now().getYear());
        //判断是出库还是入库，出库为发物单位，入库为收物单位
        if (req.getOrderType().equals("in")){
            orderNum.setOrgId(Long.valueOf(req.getEndOrgId()));
        }else {
            orderNum.setOrgId(Long.valueOf(req.getStartOrgId()));
        }
        //业务类型(简写)
        orderNum.setBussinessType(req.getBussinessBrief());
        //出入业务状态
        orderNum.setOutInType(req.getOrderType());

        OrderNum one = orderNumService.getOne(new LambdaQueryWrapper<OrderNum>()
                .eq(OrderNum::getOrgId, orderNum.getOrgId())
                .eq(OrderNum::getBussinessType, orderNum.getBussinessType())
                .eq(OrderNum::getYear, orderNum.getYear())
                .eq(OrderNum::getOutInType, orderNum.getOutInType()));
        if (ObjectUtil.isNull(one)){
            orderNum.setNum(1);
            orderNumService.save(orderNum);
            return orderNum;
        }else {
            one.setNum(one.getNum()+1);
            orderNumService.updateById(one);
            return one;
        }

    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public String AddFinishOrder(UpdateOrderReq req) {
        //添加到数据库中
        String orderId = AddOrder(req);
        //将完成的任务单直接推送到消息队列rabbitmq中（需要判断是入库单还是出库单，入库单传发物单位id，出库单传收物单位id）
        //方法1：一个交换机，一个队列。通过中间对象，object存储对象，type表标识
//        if(req.getOrderType().equals("in")){
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getStartOrgId()));
//        }else {
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getEndOrgId()));
//        }
        //方法2：一个交换机，多个队列。动态创建队列
        String exchangeName="123456";
        //需要判断是入库单还是出库单，入库单传发物单位id，出库单传收物单位id
        if(req.getOrderType().equals("in")){
            Queue queue=new Queue(req.getEndOrgId(),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getEndOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getEndOrgId(), req);

        }else {
            Queue queue=new Queue(req.getStartOrgId(),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getStartOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getStartOrgId(), req);

        }
        return orderId;
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean updateOrder(UpdateOrderReq req) {
        OrderMain order = orderMainExist(req.getId());
        if (!order.getExamineState().equals("none"))
        {
            throw new ServiceException(OrderExceptionEnum.ORDER_CAN_NOT_UPDATE);
        }
        List<OrderDetail> list= orderDetailService.QueryList(req.getId());
        //idSet中存放的是存在数据库中的任务单中的orderDetail中的id（原始的工作单中的数据id）
        Set<String> idSet =new HashSet<>();
        for (OrderDetail orderDetail:list){
            idSet.add(Long.toString(orderDetail.getId()));
        }
        //将该任务单的所有id转换为String,并添加到updateList中，用来判断删除或者修改新增的是否存在
        //判断删除列表是否为空，如果不为空则进行批量删除
        if (ObjectUtil.isNotNull(req.getDeleteList())&& ObjectUtil.isNotEmpty(req.getDeleteList())) {
            //判断传递的list Id是否存在，如果不存在则抛出异常
            for (String s:req.getDeleteList()){
                if (idSet.contains(s)){
                    //将要删除的装备价格和数据进行查询出来，并别对任务单的总数量和总价格进行更改
                    OrderDetail one = orderDetailService.getOne(new LambdaQueryWrapper<OrderDetail>()
                            .eq(OrderDetail::getOrderId, s));
                    order.setPrice(order.getPrice().subtract(one.getPrice().multiply(BigDecimal.valueOf(one.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() - one.getPlanNum());
                    orderDetailService.removeBatchByIds(req.getDeleteList());
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }
            }
        }
        //对detaillist进行处理
        List<OrderDetail> addList = new ArrayList<>();//新增的
        List<OrderDetail> updateList = new ArrayList<>();//更新
        for (DetailListReq detailListReq :req.getDetailList()) {
            //判断是否存在，存在则进行修改
            if (idSet.contains(detailListReq.getId())){
                OrderDetail one = orderDetailService.getOne(new LambdaQueryWrapper<OrderDetail>()
                        .eq(OrderDetail::getId, detailListReq.getId()));
                BeanPlusUtil.copyProperties(detailListReq,one);
                one.setType(req.getOrderType());
                //判断数量和价格是否有变动，如果有变动，则将order的总价和总数量进行更新操作，否则不变
                if (one.getPlanNum().equals(detailListReq.getPlanNum()) && one.getPrice().compareTo(detailListReq.getPrice())!=0){
                    order.setPrice(order.getPrice().subtract(one.getPrice().multiply(BigDecimal.valueOf(one.getPlanNum())))
                            .add(detailListReq.getPrice().multiply(BigDecimal.valueOf(detailListReq.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() - one.getPlanNum()+ detailListReq.getPlanNum());
                }
                updateList.add(one);
            }else {
                //否则判断id是否为空或者错误，如果为空则添加，不为空则抛出异常
                if (ObjectUtil.isNull(detailListReq.getId()) || detailListReq.getId().equals("") || ObjectUtil.isEmpty(detailListReq.getId())){
                    OrderDetail detail=new OrderDetail();
                    BeanPlusUtil.copyProperties(detailListReq, detail);
                    detail.setOrderId(req.getId());
                    detail.setType(req.getOrderType());
                    //将数量和价格进行相加处理
                    order.setPrice(order.getPrice().add(detailListReq.getPrice().multiply(BigDecimal.valueOf(detailListReq.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() + detailListReq.getPlanNum());
                    addList.add(detail);
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }

            }
        }
        orderDetailService.updateBatchById(updateList);
        orderDetailService.saveBatch(addList);
        BeanPlusUtil.copyProperties(req,order);
        this.updateById(order);
        return  true;
    }
    public boolean hasCommonElements(List<?> list1, List<?> list2) {
        for (Object item1 : list1) {
            if (list2.contains(item1)) {
                return true; // 找到相同值
            }
        }
        return false; // 没有相同值
    }



    //查询任务列表(根据组织机构id)
    @Override
    public PageResult<OrderMainDto> getOrderPage(OrderMainReq req) {
        //判断组织机构是否存在
        pubOrgService.PubOrgExist(req.getOrgId());
        //判断是出库单还是入库单，选择的查询条件也不相同
        Page<OrderMain> page=new Page<>();
        LambdaQueryWrapper<OrderMain> wrapper = createWrapper(req);
        if (req.getOrderType() != null && req.getOrderType().equals("in")){//入库单
            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType())
                    .eq(OrderMain::getEndOrgId, req.getOrgId());
            page=this.page(PageFactory.getDefaultPage(),eq);
        } else if (req.getOrderType() != null && req.getOrderType().equals("out")) {//出库单
            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType()).eq(OrderMain::getStartOrgId, req.getOrgId());
            page=this.page(PageFactory.getDefaultPage(),eq);
        }else {//查询全部
            Page<OrderMain> page1=new Page<>();
            Page<OrderMain> page2=new Page<>();
            LambdaQueryWrapper<OrderMain> eq1 = wrapper.eq(OrderMain::getEndOrgId, req.getOrgId());
            page1=this.page(PageFactory.getDefaultPage(),eq1);
            LambdaQueryWrapper<OrderMain> eq2 = wrapper.eq(OrderMain::getStartOrgId, req.getOrgId());
            page2=this.page(PageFactory.getDefaultPage(),eq2);
            List<OrderMain> list = new ArrayList<>();
            list.addAll(page1.getRecords());
            list.addAll(page2.getRecords());
            page.setRecords(list);
        }
        List<OrderMainDto> orderMainDtoList=new ArrayList<>();
        //遍历page,(如果出入库的组织机构为同一个的时候需要进行去重处理，不然会存在两条重复的数据)
        for (OrderMain orderMain : page.getRecords()){
            OrderMainDto orderMainDto = new OrderMainDto();
            BeanPlusUtil.copyProperties(orderMain,orderMainDto);
            if (!orderMainDtoList.contains(orderMainDto)){
                orderMainDtoList.add(orderMainDto);
            }
        }
        //设置pageSize，pageNo,count,list
        Map<String, Long> map = getPage(req.getPageNo(), req.getPageSize());
        Long pageNo = map.get("pageNo");
        Long pageSize = map.get("pageSize");
        long count=orderMainDtoList.size();
        return PageResultFactory.createPageResult(orderMainDtoList,count,pageSize,pageNo);
    }

    //根据任务单id查看业务明细
    @Override
    public OrderDto GetDetailById(OrderMainReq req) {
        //判断任务单是否存在
        OrderMain orderMain = orderMainExist(req.getId());
        OrderDto orderDto = new OrderDto();
        BeanPlusUtil.copyProperties(orderMain,orderDto);
        //将详细信息存入列表属性中
        //通过任务单id查询对应的详细信息
        List<OrderDetail> list = orderDetailService.list(new LambdaQueryWrapper<OrderDetail>()
                .eq(OrderDetail::getOrderId, req.getId()));
        List<OrderDetailDto> orderDetailDtoList=new ArrayList<>();
        for (OrderDetail orderDetail:list) {
            OrderDetailDto orderDetailDto = new OrderDetailDto();
            BeanPlusUtil.copyProperties(orderDetail,orderDetailDto);
            orderDetailDtoList.add(orderDetailDto);
        }
        orderDto.setDetailList(orderDetailDtoList);
        return orderDto;
    }


    //判断任务单是否存在
    @Override
    public OrderMain orderMainExist(String id) {
        OrderMain order = this.getById(id);
        if (ObjectUtil.isNull(order)) {
            throw new ServiceException(OrderExceptionEnum.ORDER_NOT_EXIST);
        }
        return order;
    }

    //设置pageNo和pageSize的通用方法
    public Map<String,Long> getPage(Long no,Long size){
        Map<String,Long> map=new HashMap<>();
        long pageSize;
        long pageNo;
        if (ObjectUtil.isNull(no)){
            pageNo = 1L;
        }else {
            pageNo= no;
        }
        if ( ObjectUtil.isNull(size)){
            pageSize = 20L;
        }else {
            pageSize= size;
        }
        map.put("pageNo",pageNo);
        map.put("pageSize",pageSize);
        return map;
    }

    private LambdaQueryWrapper<OrderMain> createWrapper(OrderMainReq req) {
        LambdaQueryWrapper<OrderMain> wrapper = new LambdaQueryWrapper<>();
        if (ObjectUtil.isEmpty(req)) {
            return wrapper;
        }
        wrapper.eq(ObjectUtil.isNotEmpty(req.getOrderState()), OrderMain::getOrderState, req.getOrderState());
        wrapper.eq(ObjectUtil.isNotEmpty(req.getExamineState()), OrderMain::getExamineState, req.getExamineState());
        wrapper.eq(ObjectUtil.isNotEmpty(req.getManualState()), OrderMain::getManualState, req.getManualState());
        return wrapper;

    }

}
